""" Fetch data from repository, or maybe local cache
"""

import os
from pathlib import Path

import pooch

DATA_VERSION='0.2'


def get_registry():
    return pooch.create(
        # Use the default cache folder for the operating system
        path=pooch.os_cache("nipraxis"),
        # The remote data is on Github
        base_url="https://raw.githubusercontent.com/nipraxis/nipraxis-data/{version}/",
        version=DATA_VERSION,
        # If this is a development version, get the data from the "main" branch
        version_dev="main",
        registry={
            'ds107_sub012_highres.nii': 'md5:316b0635a4280f65e1ca27ecb34264d6',
            'ds107_sub012_t1r2.nii': 'md5:4546a0a3f7041261b80b56b60cd54126',
            'ds114_sub009_highres.nii': 'md5:95d1b9542a9adebb87e3948c33af478d',
            'ds114_sub009_highres_brain_222.nii': 'md5:615aad84d5b96085601fe306af614564',
            'ds114_sub009_t2r1.nii': 'md5:709fcca8d33ddb7d0b7d501210c8f51c',
            'mni_icbm152_t1_tal_nlin_asym_09a_masked_222.nii': 'md5:6d0615d6581c9f9e17f6916da480fd2e',
            'camera.txt': 'md5:e596928a61c4332252d4eb1f0b6dab1e',
            'ds114_sub009_highres_moved.hdr': 'md5:b12b3d9db549d68b984ace0b95920603',
            'ds114_sub009_highres_moved.img': 'md5:95d1b9542a9adebb87e3948c33af478d',
            'ds114_sub009_t2r1_cond.txt': 'md5:5cb29aed9c9f330afe1af7e69f8aad18',
            'new_cond.txt': 'md5:b40dc95801267932a9f273f02ac05d1e',
            'ds114_sub009_t2r1_conv.txt':
            'md5:839eadd4533af9d8aef0cf8d623ab139',
        },
        # The name of an environment variable that can overwrite the cache path.
        env="NIPRAXIS_LOCAL_CACHE",
    )


NIPRAXIS_REGISTRY = get_registry()


def from_staging_cache(rel_url, staging_cache):
    if not (known_hash := NIPRAXIS_REGISTRY.registry.get(rel_url)):
        return None
    pth = Path(staging_cache).resolve() / DATA_VERSION / rel_url
    action, verb = pooch.core.download_action(pth, known_hash)
    if action == 'update':
        pooch.utils.get_logger().info(
            f"'{rel_url}' in '{staging_cache}/{DATA_VERSION}' "
            "but hash does not match; looking in local cache / registry.")
        return None
    if action == 'fetch':
        return str(pth)


def fetch_file(rel_url):
    """ Fetch data file from local cache, or registry
    """
    staging_cache = os.environ.get('NIPRAXIS_STAGING_CACHE')
    if staging_cache and (cache_fname := from_staging_cache(rel_url, staging_cache)):
        return cache_fname
    return NIPRAXIS_REGISTRY.fetch(rel_url)



""" Test local cache
"""
import shutil
from hashlib import sha1

from nipraxis import fetch_file
import nipraxis._fetcher as nipf

CAMERA_HASH = 'af7257977f30797d4b3ea7dd15fa362d4fe8c37e'


def assert_hash(fname, hash):
    with open(fname, 'rb') as fobj:
        contents = fobj.read()
    assert sha1(contents).hexdigest() == hash


def test_camera(tmp_path, monkeypatch):
    local_cache = tmp_path / 'nipraxis-local'
    staging_cache = tmp_path / 'nipraxis-staging'
    monkeypatch.delenv("NIPRAXIS_STAGING_CACHE", raising=False)
    monkeypatch.setenv("NIPRAXIS_LOCAL_CACHE", str(local_cache))
    monkeypatch.setattr('nipraxis._fetcher.NIPRAXIS_REGISTRY',  nipf.get_registry())
    fname = fetch_file('camera.txt')
    assert fname.startswith(str(local_cache))
    monkeypatch.setenv("NIPRAXIS_STAGING_CACHE", str(staging_cache))
    fname = fetch_file('camera.txt')
    assert fname.startswith(str(local_cache))
    shutil.move(local_cache, staging_cache)
    fname = fetch_file('camera.txt')
    assert fname.startswith(str(staging_cache))



""" Test stimuli module

Run tests with::

    py.test test_stimuli.py
"""

import os

import numpy as np
import numpy.testing as npt

from nipraxis import stimuli

# Example onset, duration, amplitude triplets
COND_TEST = """\
10    6.0    1
20    4.0    2
24    2.0    0.1"""

# Name of file to write for testing
COND_TEST_FNAME = 'cond_test1.txt'


def test_events2neural():
    # test events2neural function
    # Write condition test file
    with open(COND_TEST_FNAME, 'wt') as fobj:
        fobj.write(COND_TEST)
    # Read it back
    neural = stimuli.events2neural(COND_TEST_FNAME, 2, 16)
    # Expected values for tr=2, n_trs=16
    expected = np.zeros(16)
    expected[5:8] = 1
    expected[10:12] = 2
    expected[12] = 0.1
    npt.assert_array_equal(neural, expected)
    neural = stimuli.events2neural(COND_TEST_FNAME, 1, 30)
    # Expected values for tr=1, n_trs=30
    expected = np.zeros(30)
    expected[10:16] = 1
    expected[20:24] = 2
    expected[24:26] = 0.1
    npt.assert_array_equal(neural, expected)
    # Remove test file
    os.unlink(COND_TEST_FNAME)



""" Functions to work with standard OpenFMRI stimulus files

The functions have docstrings according to the numpy docstring standard - see:

    https://github.com/numpy/numpy/blob/master/doc/HOWTO_DOCUMENT.rst.txt
"""

import numpy as np

def events2neural(task_fname, tr, n_trs):
    """ Return predicted neural time course from event file `task_fname`

    Parameters
    ----------
    task_fname : str
        Filename of event file
    tr : float
        TR in seconds
    n_trs : int
        Number of TRs in functional run

    Returns
    -------
    time_course : array shape (n_trs,)
        Predicted neural time course, one value per TR
    """
    task = np.loadtxt(task_fname)
    # Check that the file is plausibly a task file
    if task.ndim != 2 or task.shape[1] != 3:
        raise ValueError("Is {0} really a task file?", task_fname)
    # Convert onset, duration seconds to TRs
    task[:, :2] = task[:, :2] / tr
    ons_dur = task[:, :2]
    if np.any(ons_dur != np.round(ons_dur)):
        raise ValueError('onsets, durations should be multiples of TR')
    # Neural time course from onset, duration, amplitude for each event
    time_course = np.zeros(n_trs)
    for onset, duration, amplitude in task:
        # Make onset and duration integers
        onset = int(round(onset))
        duration = int(round(duration))
        time_course[onset:onset + duration] = amplitude
    return time_course
